@@ -103,6 +103,7 @@ class CHMInput(InputFormatPlugin):
         from calibre.customize.builtins import HTMLInput
         opts.breadth_first = True
         htmlinput = HTMLInput(None)
+        htmlinput.set_root_dir_of_input(basedir)
         oeb = htmlinput.create_oebbook(htmlpath, basedir, opts, log, mi)
         return oeb
 
@@ -172,7 +173,7 @@ class CHMInput(InputFormatPlugin):
         return htmlpath, toc
 
     def _read_file(self, name):
-        with lopen(name, 'rb') as f:
+        with open(name, 'rb') as f:
             data = f.read()
         return data
 
