@@ -14,10 +14,14 @@ import time
 import traceback
 from collections import defaultdict, namedtuple
 from itertools import groupby
+from qt.core import (
+    QAbstractTableModel, QApplication, QColor, QDateTime, QFont, QFontMetrics, QIcon,
+    QImage, QModelIndex, QPainter, QPixmap, Qt, pyqtSignal,
+)
 
 from calibre import (
     fit_image, force_unicode, human_readable, isbytestring, prepare_string_for_xml,
-    strftime
+    strftime,
 )
 from calibre.constants import DEBUG, config_dir, dark_link_color, filesystem_encoding
 from calibre.db.search import CONTAINS_MATCH, EQUALS_MATCH, REGEXP_MATCH, _match
@@ -30,15 +34,14 @@ from calibre.library.coloring import color_row_key
 from calibre.library.save_to_disk import find_plugboard
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.config import device_prefs, prefs, tweaks
-from calibre.utils.date import (UNDEFINED_DATE, as_local_time, dt_factory,
-                                is_date_undefined, qt_to_dt)
+from calibre.utils.date import (
+    UNDEFINED_DATE, as_local_time, dt_factory, is_date_undefined, qt_to_dt,
+)
 from calibre.utils.icu import sort_key
-from calibre.utils.localization import calibre_langcode_to_name
+from calibre.utils.localization import calibre_langcode_to_name, ngettext
+from calibre.utils.resources import get_path as P
 from calibre.utils.search_query_parser import ParseException, SearchQueryParser
 from polyglot.builtins import iteritems, itervalues, string_or_bytes
-from qt.core import (QAbstractTableModel, QApplication, QColor, QDateTime,
-                     QFont, QFontMetrics, QIcon, QImage, QModelIndex, QPainter,
-                     QPixmap, Qt, pyqtSignal)
 
 Counts = namedtuple('Counts', 'library_total total current')
 
@@ -202,8 +205,10 @@ class BooksModel(QAbstractTableModel):  # {{{
                         'series'    : ngettext("Series", 'Series', 1),
                         'last_modified' : _('Modified'),
                         'languages' : _('Languages'),
+                        'formats'   : _('Formats'),
+                        'id'        : _('Id'),
+                        'path'      : _('Path'),
         }
-
         self.db = None
 
         self.formatter = SafeFormat()
@@ -400,6 +405,8 @@ class BooksModel(QAbstractTableModel):  # {{{
     def refresh_rows(self, rows, current_row=-1):
         self._clear_caches()
         cc = self.columnCount(QModelIndex()) - 1
+        for r in rows:
+            self.db.new_api.clear_extra_files_cache(self.db.id(r))
         for first_row, last_row in group_numbers(rows):
             self.dataChanged.emit(self.index(first_row, 0), self.index(last_row, cc))
             if current_row >= 0 and first_row <= current_row <= last_row:
@@ -580,6 +587,11 @@ class BooksModel(QAbstractTableModel):  # {{{
         if not self.db:
             return
         self.db.multisort(self.sort_history[:tweaks['maximum_resort_levels']])
+        if self.sort_history:
+            # There are ways to get here that don't set sorted_on, e.g., the
+            # sort_by action. I don't think sort_history can be empty, but it
+            # doesn't hurt to check
+            self.sorted_on = self.sort_history[0]
         if reset:
             self.beginResetModel(), self.endResetModel()
 
@@ -821,6 +833,10 @@ class BooksModel(QAbstractTableModel):  # {{{
 
         def renderer(field, decorator=False):
             idfunc = self.db.id
+            if field == 'id':
+                def func(idx):
+                    return idfunc(idx)
+                return func
             fffunc = self.db.new_api.fast_field_for
             field_obj = self.db.new_api.fields[field]
             m = field_obj.metadata.copy()
@@ -945,7 +961,7 @@ class BooksModel(QAbstractTableModel):  # {{{
 
             return func
 
-        self.dc = {f:renderer(f) for f in 'title authors size timestamp pubdate last_modified rating publisher tags series ondevice languages'.split()}
+        self.dc = {f:renderer(f) for f in self.orig_headers.keys()}
         self.dc_decorator = {f:renderer(f, True) for f in ('ondevice',)}
 
         for col in self.custom_columns:
@@ -1599,7 +1615,7 @@ class DeviceBooksModel(BooksModel):  # {{{
                 return ax
 
         keygen = {
-                'title': ('title_sorter', lambda x: sort_key(x) if x else ''),
+                'title': ('title_sorter', lambda x: sort_key(x) if x else b''),
                 'authors' : author_key,
                 'size' : ('size', int),
                 'timestamp': ('datetime', functools.partial(dt_factory, assume_utc=True)),
@@ -1698,6 +1714,10 @@ class DeviceBooksModel(BooksModel):  # {{{
     def current_changed(self, current, previous, emit_signal=True):
         if current.isValid():
             idx = current.row()
+            try:
+                self.db[self.map[idx]]
+            except Exception:
+                return  # can happen if the device is ejected
             try:
                 data = self.get_book_display_info(idx)
             except Exception:
@@ -1713,6 +1733,13 @@ class DeviceBooksModel(BooksModel):  # {{{
     def paths(self, rows):
         return [self.db[self.map[r.row()]].path for r in rows]
 
+    def id(self, row):
+        row = getattr(row, 'row', lambda:row)()
+        try:
+            return self.db[self.map[row]].path
+        except Exception:
+            return None
+
     def paths_for_db_ids(self, db_ids, as_map=False):
         res = defaultdict(list) if as_map else []
         for r,b in enumerate(self.db):
